#include <stdio.h>
#include <stdbool.h>   /* for c99 bool */

#include "armv8_aarch64_pmu.h"


static unsigned int dot_product(unsigned char *a, unsigned char *b, int size);
static bool dot_product_supported();

extern void init_serial(void);

extern unsigned int dot_product_a55(unsigned char *a, unsigned char *b, int size);

static unsigned char a[64] = {1, 3, 5, 6, 1, 3, 5, 6, 1, 3, 5, 6, 1, 3, 5, 6, 1, 3, 5, 6, 1, 3, 5, 6, 1, 3, 5, 6, 1, 3, 5, 6, 1, 3, 5, 6, 1, 3, 5, 6, 1, 3, 5, 6, 1, 3, 5, 6, 1, 3, 5, 6, 1, 3, 5, 6, 1, 3, 5, 6, 1, 3, 5, 6};

static unsigned char b[64]= {4, 2, 1, 3, 4, 2, 1, 3, 1, 3, 5, 6, 1, 3, 5, 6, 1, 3, 5, 6, 1, 3, 5, 6, 1, 3, 5, 6, 1, 3, 5, 6, 1, 3, 5, 6, 1, 3, 5, 6, 1, 3, 5, 6, 1, 3, 5, 6, 1, 3, 5, 6, 1, 3, 5, 6, 1, 3, 5, 6, 1, 3, 5, 6 };


static unsigned long long read_id_aa64isar0()
{
    unsigned long long id_aa64isar0;

    __asm ("MRS %x0, ID_AA64ISAR0_EL1 \n" : "=r" (id_aa64isar0) );

    return (id_aa64isar0);
}

static bool dot_product_supported()
{
    if (read_id_aa64isar0() & 0x0000100000000000ULL)
        return true;
    else 
        return false;
}

static void start_cycle_count()
{
    resetCCNT();
    enableCCNT();
    enablePMU();
}

static int end_cycle_count()
{
    unsigned int count;

    disablePMU();
    count = readCCNT();

    return (count);
}

__attribute__((noreturn)) int main()
{
    unsigned int res1;
    unsigned int res2;
    unsigned int ccnt1, ccnt2;

    (void) init_serial();

    printf("Starting 64x64 DOT Product test....\n");

    (void) start_cycle_count();

    res1 = dot_product(a, b, (int) 64);

    ccnt1 = end_cycle_count();

    printf("Computed Result: %d\n", res1);
    printf("Number of cycles: %d\n\n\n", ccnt1);

    printf("Starting 64x64 Cortex-A55 DOT Product test....\n");
   
    if (dot_product_supported())
    {

        (void) start_cycle_count();

        res2 = dot_product_a55(a, b, (int) 64);

        ccnt2 = end_cycle_count();

        printf("Result: %d\n", res2);
        printf("Number of cycles: %d\n\n\n", ccnt2);

    }
    else
        printf("DOT Product not available....\n");

    while(1);

}


unsigned __attribute__((noinline)) dot_product(unsigned char *a, unsigned char *b, int size)
{
    unsigned int sum = 0;

    for (int i = 0; i < size; i++) {
        sum += a[i] * b[i];
    }

    return sum;
}

