/*-----------------------------------------------------------------------------
// The confidential and proprietary information contained in this file may
// only be used by a person authorised under and to the extent permitted
// by a subsisting licensing agreement from ARM Limited.
//
//            (C) COPYRIGHT 1994-2007 ARM Limited.
//                ALL RIGHTS RESERVED
//
// This entire notice must be reproduced on all copies of this file
// and copies of this file may only be made by a person if such person is
// permitted to do so under the terms of a subsisting license agreement
// from ARM Limited.
//---------------------------------------------------------------------------*/

/******************************************************************************
// Copyright (c) 2003-2014 by Carbon Design Systems, Inc., All Rights
// Reserved.
//
// THIS SOFTWARE CONTAINS CONFIDENTIAL INFORMATION AND TRADE SECRETS OF CARBON
// DESIGN SYSTEMS, INC.  PERMISSION IS HEREBY GRANTED TO USE, MODIFY, AND/OR
// COPY THIS SOFTWARE FOR INTERNAL USE ONLY PROVIDED THAT THE ABOVE COPYRIGHT
// NOTICE APPEARS IN ALL COPIES OF THIS SOFTWARE.
******************************************************************************/


#include <stdio.h>
//#include <rt_misc.h>

#include "uart.h"

#define TRUE 1
#define FALSE 0


/*
** Importing __use_no_semihosting ensures that our image doesn't link
** with any C Library code that makes direct use of semihosting.
**
*/
//asm("  .global __use_no_semihosting\n");


void init_serial(void)
{
    /* Disable the serial port while setting the baud rate and word length */
    UART0_CR = 0;

    /* Setting the correct baud rate and word length */
    UART0_LCRL = LCRL_Baud_460800;      // LCRL and LCRM writes _MUST_
                                        // be performed before the LCRH
    UART0_LCRM = LCRM_Baud_460800;      // write as LCRH generates the
                                        // write strobe to transfer the
    UART0_LCRH = LCRH_Word_Length_8;    // data. FIFO disabled.

    /* Now enable the serial port with no interrupts */
    UART0_CR   = CR_UART_Enable | CR_TX_Int_Enable | CR_RX_Int_Enable;
}

void sendchar(unsigned char *ch)
{
    while (UART0_FR & FR_TX_Fifo_Full)
        ;

    UART0_DR = *ch;                     // Transmit next character
    if (*ch == '\n')                    // Replace line feed with '\r'
    {
        *ch = '\r';                   // Replace line feed with '\r'
        sendchar(ch);
    }
}

unsigned char receive_char(void)
{
    volatile char status;
    volatile char ch = '0';

    while (UART0_FR & FR_RX_Fifo_Empty) // wait until we receive a byte
        ;

    ch = UART0_DR;         // receive character
    status = UART0_RSR;    // read the status register

    /* Could check status receive status here */
    return ch;
}


/*
** Retargeted I/O
** ==============
** The following C library functions make use of semihosting
** to read or write characters to the debugger console: fputc(),
** fgetc(), and _ttywrch().  They must be retargeted to write to
** the Integrator AP UART.  __backspace() must also be retargeted
** with this layer to enable scanf().  See the Compiler and
** Libraries Guide.
*/

/*
** These must be defined to avoid linking in stdio.o from the
** C Library
*/

struct __FILE { int handle;   /* Add whatever you need here */};
FILE __stdout;
FILE __stdin;
FILE __stderr;


/* UART access functions */
void init_serial(void);
void sendchar(unsigned char *ch);
unsigned char receive_char(void);

/*
** __backspace must return the last char read to the stream
** fgetc() needs to keep a record of whether __backspace was
** called directly before it
*/
int last_char_read;
int backspace_called;

int fputc(int ch, FILE *f)
{
    unsigned char tempch = ch;

    sendchar(&tempch);

    return ch;
}

int fgetc(FILE *f)
{
    unsigned char tempch;

    /* if we just backspaced, then return the backspaced character */
    /* otherwise output the next character in the stream */
    if (backspace_called == TRUE)
    {
        backspace_called = FALSE;
        return last_char_read;
    }

    tempch = receive_char();

    last_char_read = (int)tempch;       /* backspace must return this value */

    fputc(tempch, f);
    if (tempch == '\r')
    {
        fputc('\n', f);
    }

    return tempch;
}

void _ttywrch(int ch)
{
    unsigned char tempch = ch;

    sendchar(&tempch);
}

/*
** The effect of __backspace() should be to return the last character
** read from the stream, such that a subsequent fgetc() will
** return the same character again.
*/

int __backspace(FILE *f)
{
    backspace_called = TRUE;
    return 0;
}

/* END of Retargeted I/O */


/*
** Exception Signaling and Handling
** ================================
** The C library implementations of ferror() uses semihosting directly
** and must therefore be retargeted.  This is a minimal reimplementation.
** _sys_exit() is called after the user's main() function has exited.  The C library
** implementation uses semihosting to report to the debugger that the application has
** finished executing.
*/


int ferror(FILE *f)
{
    return EOF;
}

void _sys_exit(int return_code)
{
    while(1);
}

char *_sys_command_string(char *cmd, int len)
{
    return NULL;
}
