// ------------------------------------------------------------
// Juno SMP Startup Code Example
//
// Copyright (c) 2011-2014 ARM Ltd.  All rights reserved.
//------------------------------------------------------------


// ------------------------------------------------------------

.equ AArch32_Mode_USR, 0x10
.equ AArch32_Mode_FIQ, 0x11
.equ AArch32_Mode_IRQ, 0x12
.equ AArch32_Mode_SVC, 0x13
.equ AArch32_Mode_ABT, 0x17
.equ AArch32_Mode_UNDEF, 0x1B
.equ AArch32_Mode_SYS, 0x1F
.equ AArch32_Mode_HYP, 0x1A
.equ AArch32_Mode_MON, 0x16

.equ AArch64_EL2_SP2, 0x09 // EL2h
.equ AArch64_EL2_SP0, 0x08 // EL2t
.equ AArch64_EL1_SP1, 0x05 // EL1h
.equ AArch64_EL1_SP0, 0x04 // EL1t
.equ AArch64_EL0_SP0, 0x00

.equ AArch32_State_Thumb, 0x20
.equ AArch32_State_ARM, 0x00

// ------------------------------------------------------------

.equ TT_S1_TABLE, 0x00000000000000003 // NSTable=0, PXNTable=0, UXNTable=0, APTable=0

// TT block entries templates   (L1 and L2, NOT L3)    
// Assuming table contents:
// 0 = b01000100 = Normal, Inner/Outer Non-Cacheable
// 1 = b11111111 = Normal, Inner/Outer WB/WA/RA
// 2 = b00000000 = Device-nGnRnE
.equ TT_S1_FAULT, 0x0
.equ TT_S1_NORMAL_NO_CACHE, 0x00000000000000401 // Index = 0, AF=1
.equ TT_S1_NORMAL_WBWA, 0x00000000000000405 // Index = 1, AF=1
.equ TT_S1_DEVICE_nGnRnE, 0x00000000000000409 // Index = 2, AF=1

.equ TT_S1_UXN, (1 << 54)
.equ TT_S1_PXN, (1 << 53)
.equ TT_S1_nG, (1 << 11)
.equ TT_S1_NS, (1 << 5)

.equ TT_S1_NON_SHARED, (0 << 8) // Non-shareable
.equ TT_S1_INNER_SHARED, (3 << 8) // Inner-shareable
.equ TT_S1_OUTER_SHARED, (2 << 8) // Outer-shareable

.equ TT_S1_PRIV_RW, (0x0)
.equ TT_S1_PRIV_RO, (0x2 << 6)
.equ TT_S1_USER_RW, (0x1 << 6)
.equ TT_S1_USER_RO, (0x3 << 6)

.equ TT_FLASH, TT_S1_NORMAL_WBWA | TT_S1_INNER_SHARED | TT_S1_PRIV_RO
.equ TT_RAM, TT_S1_NORMAL_WBWA | TT_S1_INNER_SHARED | TT_S1_PRIV_RW
.equ TT_PERIPHERAL, TT_S1_DEVICE_nGnRnE | TT_S1_PXN | TT_S1_UXN | TT_S1_PRIV_RW

// ------------------------------------------------------------
// Imports
// ------------------------------------------------------------

// ------------------------------------------------------------


    .section  STARTUP, "ax"
    .align 8
    .global start64
    .type start64, @function
start64:


	.global Image$$ARM_LIB_STACK$$ZI$$Base
  LDR      x1, Image$$ARM_LIB_STACK$$ZI$$Base     // Get address of level 1 for TTBR0_EL1
  MOV     sp, x1


  //
  // Configure SCR_EL3
  // ------------------
  // Route IRQs to EL3
  MOV      x1, xzr
  ORR      x1, x1, #2                     // Set IRQ bit
  ORR      x1, x1, #1                     // Set NS bit
  MSR      SCR_EL3, x1

  // Disable trapping of CPTR_EL3 accesses or use of Adv.SIMD/FPU
  // -------------------------------------------------------------
  MSR      CPTR_EL3, xzr
  ISB

  // Set the Base address
  // ---------------------

  LDR      x1, =ttb0_base
  MSR      TTBR0_EL3, x1                    // Set TTBR0

  // Set up memory attributes
  // -------------------------
  // This equates to:
  // 0 = b01000100 = Normal, Inner/Outer Non-Cacheable
  // 1 = b11111111 = Normal, Inner/Outer WB/WA/RA
  // 2 = b00000000 = Device-nGnRnE
  LDR      x1, =0x0000000000FF0404
  MSR      MAIR_EL3, x1

  // Set up TCR_EL1
  // ---------------
  // EPD0=0b0      Table walks using TTBR0 enabled
  // EPD1=0b1      Table walks using TTBR1 disabled
  // TBI*=0b0      Top byte not ignored
  // TG0=0b00      4KB granule
  // T0SZ=0b011001 Limits VA space to 39 bits (38:0), translation starts @ l1
  // SH0=0b11      Inner Shareable
  // IGRN0=0b01    Walks to TTBR0 are Inner WB/WA
  // OGRN0=0b01    Walks to TTBR0 are Outer WB/WA
  // AS=0          8-bit ASID
  // IPS=b001      Physical address size is 36-bit
  LDR      x1, =0x0000000135993520
  MSR      TCR_EL3, x1

  // Ensure changes to system register are visible before MMU enabled
  ISB

  // Which core am I
  // ----------------
  MRS      x0, MPIDR_EL1
  ANDS     x0, x0, #0xFFFF                   // Mask off to leave Aff0/1
  CBZ      x0, primaryCPUInit                // If core 0, run the primary init code
  CBNZ     x0, secondaryCPUsInit


primaryCPUInit:	


  // Enable MMU
  // -----------
  MRS      x1, SCTLR_EL3
  ORR      x1, x1, #(1 << 0)                 // Set M bit
  ORR      x1, x1, #(1 << 2)                 // Set the SCTLR_EL3.C bit
  ORR      x1, x1, #(1 << 12)                 // Set the SCTLR_EL3.I bit
  MSR      SCTLR_EL3, x1
  ISB

  B         __main


secondaryCPUsInit:
  WFE
  B         secondaryCPUsInit




    .global dot_product_a55_8x8
    .type dot_product_a55_8x8, "function"
dot_product_a55_8x8:

  LDR  D1, [x0]
  LDR  D2, [x1]

  DUP  v0.2d, xzr
  UDOT V0.2S, V1.8B, V2.8B
  UMOV  x0, v0.d[0]

  FMOV  x0, d0

  RET

      .global dot_product_a55_4x4
    .type dot_product_a55_4x4, "function"
dot_product_a55_4x4:

  LDR  S1, [x0]
  LDR  S2, [x1]

  DUP  v0.2d, xzr
  UDOT V0.2S, V1.8B, V2.8B

  FMOV  w0, s0

  RET


    .global dot_product_a55_16x16
    .type dot_product_a55_16x16, "function"
dot_product_a55_16x16:

  MOV   x3, xzr
  DUP   v0.2d, x3

  LD1   {v1.2d}, [x0]
  LD1   {v2.2d}, [x1]

  UDOT  v0.4s, v1.16b, v2.16b
  ADDV  s0, v0.4s

  UMOV  x0, v0.d[0]

  RET



    .global dot_product_a55
    .type dot_product_a55, "function"
// x0 - unsigned char source pointer 1
// x1 - unsigned char source pointer 2
// x2 - vector size  - must be multiple of 16
dot_product_a55:

  ASR   x2, x2, #4       // compute loop count
  MOV   x3, xzr
  DUP   v0.2d, x3        // clear out destination vector

nextblock:
  LD1 {v1.2d}, [x0], #0x10
  LD1   {v2.2d}, [x1], #0x10
  UDOT  v0.4s, v1.16b, v2.16b
  SUB   x2, x2, #1
  CBNZ  x2, nextblock

  // add the four individual dot products
  ADDV  s0, v0.4s

  // return results in r0
  UMOV  x0, v0.d[0]

  RET









 
// ------------------------------------------------------------
// End of file
// ------------------------------------------------------------

